"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.PagesBuilder = void 0;

var _discord = require("discord.js");

var _types = require("./types");

var _utils = require("./utils");

class PagesBuilder extends _discord.EmbedBuilder {
  constructor(interaction) {
    super();
    this.interactionId = '';
    /**
     * Pages
     */

    this.pages = [];
    this.files = [];
    this.currentPage = 1;
    this.paginationFormat = '%c / %m';
    this.loop = true;
    /**
     * Components
     */

    this.components = [];
    this.defaultButtons = [];
    /**
     * Listen
     */

    this.listenTimeout = 5 * 60 * 1000;
    this.autoResetTimeout = true;
    this.endColor = 'Grey';
    this.endMethod = _types.EndMethod.EDIT;
    /**
     * Triggers
     */

    this.triggers = new Map();
    this.interaction = interaction;
    this.listenUsers = [interaction.member.user.id];
    this.setDefaultButtons();
  }
  /**
   * Method for initial pages setup
   *
   * @example
   * ```
   * builder.setPages(
   *     new Embed()
   *         .setTitle('Hello World!')
   * );
   *
   * builder.setPages([
   *     new Embed()
   *         .setTitle('Hello World!'),
   *     () => (
   *         new Embed()
   *             .setTitle('Function page')
   *     )
   * ]);
   * ```
   */


  setPages(pages) {
    if (!Array.isArray(pages)) {
      pages = [pages];
    }

    this.pages = pages;
    return this;
  }
  /**
   * Method for adding pages to the end
   */


  addPages(pages) {
    this.pages = this.pages.concat(pages);
    return this;
  }
  /**
   * Method for initial files setup
   */


  setFiles(files) {
    if (!Array.isArray(files)) {
      files = [files];
    }

    this.files = files;
    return this;
  }
  /**
   * Method for adding files to the end
   */


  addFiles(files) {
    this.files = this.files.concat(files);
    return this;
  }
  /**
   * Method for auto generating pages
   *
   * @example
   * ```
   * builder.autoGeneratePages({
   *     items: [
   *         'Player 1',
   *         'Player 2',
   *         'Player 3'
   *     ],
   *     countPerPage: 2
   * });
   * ```
   */


  autoGeneratePages({
    items,
    countPerPage = 10
  }) {
    const chunks = (0, _utils.chunk)(items, countPerPage);
    this.setPages(chunks.map(chunk => new _discord.EmbedBuilder().setDescription(chunk.join('\n'))));
    return this;
  }
  /**
   * Method for opening a specific page
   */


  async setPage(pageNumber) {
    this.currentPage = pageNumber;
    const data = {
      embeds: await this.getPage(pageNumber),
      components: this.simplifyKeyboard([...this.defaultButtons, ...this.components])
    };

    if (this.messageComponent) {
      const method = this.messageComponent.deferred || this.messageComponent.replied ? 'editReply' : 'update';
      return this.messageComponent[method](data);
    }

    if (this.buildMethod !== 'reply') {
      return this.message.edit(data);
    }

    return this.interaction.editReply(data);
  }
  /**
   * Method for getting the page
   */


  async getPage(pageNumber = this.currentPage) {
    let page = this.pages[pageNumber - 1];

    if (typeof page === 'function') {
      page = await page();
    }

    const resultPage = new _discord.EmbedBuilder((0, _utils.mergeDeep)(this.toJSON(), page.toJSON()));

    if (this.paginationFormat) {
      const pageNumber = this.paginationFormat.replace('%c', String(this.currentPage)).replace('%m', String(this.pages.length));
      resultPage.setFooter({ ...resultPage.data.footer,
        text: pageNumber
      });
    }

    return [resultPage];
  }
  /**
   * Method for setting the pagination format
   *
   * @example
   * %c - Current page
   * %m - Max page
   *
   * ```
   * builder.setPaginationFormat('Current page: %c, Max: %m');
   * ```
   */


  setPaginationFormat(format = '%c / %m') {
    this.paginationFormat = format;
    return this;
  }
  /**
   * Method for setting endless page switching when reaching the end
   */


  setLoop(status = true) {
    this.loop = status;
    return this;
  }
  /**
   * Method for setting default buttons
   *
   * @example
   * ```
   * builder.setDefaultButtons(['first', {
   *   stop: new ButtonComponent()
   *      .setLabel('Stop')
   *      .setStyle('PRIMARY')
   * }]);
   * ```
   */


  setDefaultButtons(buttons = [_types.Action.FIRST, _types.Action.BACK, _types.Action.STOP, _types.Action.NEXT, _types.Action.LAST]) {
    const defaultActions = new Map([[_types.Action.FIRST, _types.ActionLabel.FIRST], [_types.Action.BACK, _types.ActionLabel.BACK], [_types.Action.STOP, _types.ActionLabel.STOP], [_types.Action.NEXT, _types.ActionLabel.NEXT], [_types.Action.LAST, _types.ActionLabel.LAST]]);
    this.defaultButtons = buttons.length ? [new _discord.ActionRowBuilder().addComponents(buttons.map(button => {
      if (typeof button === 'string') {
        return new _discord.ButtonBuilder().setCustomId(button).setEmoji(defaultActions.get(button)).setStyle(_discord.ButtonStyle.Secondary);
      }

      const [[buttonAction, buttonComponent]] = Object.entries(button);
      return buttonComponent.setCustomId(buttonAction);
    }))] : [];
    return this;
  }
  /**
   * Method for setting the time to listen for updates to switch pages
   */


  setListenTimeout(timeout = 5 * 60 * 1000) {
    this.listenTimeout = timeout;
    return this;
  }
  /**
   * @description Method for resetting the current listening timer
   */


  resetListenTimeout({
    isFirstBuild = false
  } = {}) {
    if (this.timeout || isFirstBuild) {
      clearTimeout(this.timeout);
      this.timeout = setTimeout(this.stopListen.bind(this), this.listenTimeout);
    }
  }
  /**
   * Method for setting embed color at the end of listening
   */


  setListenEndColor(color = 'Grey') {
    this.endColor = color;
    return this;
  }
  /**
   * Method for setting the method of working with a message when you finish listening for reactions
   */


  setListenEndMethod(method) {
    this.endMethod = method;
    return this;
  }
  /**
   * Method for setting listening to specific users
   */


  setListenUsers(users = []) {
    this.listenUsers = !Array.isArray(users) ? [users] : users;
    return this;
  }
  /**
   * Method for adding listening to specific users
   */


  addListenUsers(users) {
    this.listenUsers = this.listenUsers.concat(users);
    return this;
  }
  /**
   * Method for setting the timer to automatically reset when switching between pages
   */


  setAutoResetTimeout(status = true) {
    this.autoResetTimeout = status;
    return this;
  }
  /**
   * Method for early stopping listening
   */


  stopListen() {
    if (this.timeout) {
      clearTimeout(this.timeout);
      this.collector.stop();
    }
  }
  /**
   * Method for setting components rows
   */


  setComponents(components) {
    if (!Array.isArray(components)) {
      components = [components];
    }

    this.components = components;
    return this;
  }
  /**
   * Method for adding components to available row
   */


  addComponents(components) {
    if (!Array.isArray(components)) {
      components = [components];
    }

    components.forEach(component => {
      if (component instanceof _discord.ActionRowBuilder) {
        return this.components.push(component);
      }

      let length = this.components.length;

      if (!length) {
        this.components.push(new _discord.ActionRowBuilder());
        length++;
      }

      const row = this.components[length - 1];

      if (component.data.type !== _discord.ComponentType.SelectMenu && row.components.length < 5 && row.components.findIndex(({
        data: {
          type
        }
      }) => type === _discord.ComponentType.SelectMenu) === -1) {
        row.addComponents(component);
      } else {
        this.components.push(new _discord.ActionRowBuilder().addComponents(component));
      }
    });
    return this;
  }
  /**
   * Method for update existing components
   *
   * @example
   * ```
   * const button = new ButtonComponent()
   *    .setCustomId('test')
   *    .setLabel('Test button')
   *    .setStyle('PRIMARY');
   *
   * builder.addComponents(button);
   *
   * button.setLabel('Primary button');
   *
   * builder.updateComponents(button);
   *
   * builder.rerender();
   * ```
   */


  updateComponents(rows) {
    if (!Array.isArray(rows)) {
      rows = [rows];
    }

    rows.forEach(component => {
      this.components = this.components.map(row => {
        // @ts-ignore
        const index = row.components.findIndex(({
          data: {
            custom_id
          }
        }) => {
          var _component$data;

          return custom_id === ((_component$data = component.data) === null || _component$data === void 0 ? void 0 : _component$data.custom_id);
        });

        if (index !== -1) {
          row.components.splice(index, 1, ...component.components);
        }

        return row;
      });
    });
    return this;
  }

  get appendedComponents() {
    return this.components;
  }
  /**
   * Method for initial setting of triggers
   */


  setTriggers(triggers = []) {
    if (!Array.isArray(triggers)) {
      triggers = [triggers];
    }

    this.triggers = new Map(triggers.map(({
      name,
      callback
    }) => [name, callback]));
    return this;
  }
  /**
   * Method for adding triggers
   */


  addTriggers(triggers) {
    if (!Array.isArray(triggers)) {
      triggers = [triggers];
    }

    triggers.forEach(({
      name,
      callback
    }) => this.triggers.set(name, callback));
    return this;
  }
  /**
   * Rerender current page
   */


  rerender() {
    return this.setPage(this.currentPage);
  }
  /**
   * Build method
   */


  async build() {
    if (this.pages.length === 0) {
      throw new TypeError('Pages not set');
    }

    const method = this.interaction.replied ? 'followUp' : this.interaction.deferred ? 'editReply' : 'reply';
    const response = await this.interaction[method]({
      embeds: await this.getPage(),
      files: this.files,
      components: this.simplifyKeyboard([...this.defaultButtons, ...this.components])
    }).then(message => {
      if (message) {
        var _message$interaction;

        const interactionId = (_message$interaction = message.interaction) === null || _message$interaction === void 0 ? void 0 : _message$interaction.id;

        if (interactionId) {
          this.interactionId = interactionId;
        }

        this.message = message;
      }
    });
    this.buildMethod = method;

    if (!this.interactionId) {
      await this.interaction.fetchReply().then(message => {
        var _message$interaction2;

        const interactionId = (_message$interaction2 = message.interaction) === null || _message$interaction2 === void 0 ? void 0 : _message$interaction2.id;

        if (interactionId) {
          this.interactionId = interactionId;
        }

        this.message = message;
      });
    }

    this.startCollector();
    this.resetListenTimeout({
      isFirstBuild: true
    });
    return response;
  }
  /**
   * @hidden
   */


  simplifyKeyboard(rows) {
    if (this.loop && this.pages.length > 1) {
      return rows.map(row => row.toJSON());
    }

    return rows.reduce((rows, row) => {
      const components = row.components.filter(({
        data
      }) => {
        // @ts-ignore
        const {
          type,
          custom_id
        } = data;
        return type !== _discord.ComponentType.Button || (this.currentPage !== 1 || this.pages.length !== 1 || custom_id !== _types.Action.FIRST && custom_id !== _types.Action.BACK) && (this.currentPage !== this.pages.length || this.pages.length !== 1 || custom_id !== _types.Action.LAST && custom_id !== _types.Action.NEXT);
      });
      row = new _discord.ActionRowBuilder({ ...row,
        components
      });

      if (row.components.length) {
        rows.push(row.toJSON());
      }

      return rows;
    }, []);
  }
  /**
   * @hidden
   */


  startCollector() {
    this.collector = this.interaction.channel.createMessageComponentCollector().on('collect', event => {
      var _event$message, _event$message$intera;

      if (((_event$message = event.message) === null || _event$message === void 0 ? void 0 : (_event$message$intera = _event$message.interaction) === null || _event$message$intera === void 0 ? void 0 : _event$message$intera.id) !== this.interactionId) {
        return;
      }

      if (this.listenUsers.length) {
        var _event$member;

        if (!this.listenUsers.includes((_event$member = event.member) === null || _event$member === void 0 ? void 0 : _event$member.user.id)) {
          return;
        }
      }

      if (this.autoResetTimeout) {
        this.resetListenTimeout();
      }

      this.messageComponent = event;
      this.message = event.message;

      if (event.isButton()) {
        this.handleButton(event);
      }

      if (event.isSelectMenu()) {
        this.handleSelectMenuComponent(event);
      }
    }).on('end', async () => {
      switch (this.endMethod) {
        case _types.EndMethod.EDIT:
          {
            const embeds = await this.getPage();
            const [embed] = embeds;
            embed.setColor(this.endColor);
            this.message.edit({
              embeds,
              components: []
            }).catch(() => null);
            break;
          }

        case _types.EndMethod.REMOVE_COMPONENTS:
          this.message.edit({
            components: []
          }).catch(() => null);
          break;

        case _types.EndMethod.REMOVE_EMBEDS:
          this.message.edit({
            embeds: []
          }).catch(() => null);
          break;

        case _types.EndMethod.DELETE:
          this.message.delete().catch(() => null);
          break;
      }
    });
  }
  /**
   * @hidden
   */


  async handleButton(interaction) {
    const {
      customId
    } = interaction;
    this.executeAction(customId);
    const trigger = this.triggers.get(customId);
    const buttons = [];

    if (trigger) {
      this.components.forEach(component => {
        component.components.forEach(button => {
          var _button$data;

          // @ts-ignore
          if (((_button$data = button.data) === null || _button$data === void 0 ? void 0 : _button$data.custom_id) === customId) {
            buttons.push(button);
          }
        });
      });
      await interaction.deferUpdate();
      await trigger(interaction, ...buttons);
      this.rerender();
    }
  }
  /**
   * @hidden
   */


  async handleSelectMenuComponent(interaction) {
    const {
      customId
    } = interaction;
    const trigger = this.triggers.get(customId);
    const menu = [];

    if (trigger) {
      this.components.forEach(component => {
        component.components.forEach(component => {
          var _component$data2;

          // @ts-ignore
          if (((_component$data2 = component.data) === null || _component$data2 === void 0 ? void 0 : _component$data2.custom_id) === customId) {
            menu.push(component);
          }
        });
      });
      await interaction.deferUpdate();
      await trigger(interaction, ...menu);
    }
  }
  /**
   * Method for invoking quick actions
   */


  executeAction(action) {
    switch (action) {
      case _types.Action.FIRST:
        if (this.currentPage === 1) {
          if (this.loop) {
            this.setPage(this.pages.length);
          }

          return;
        }

        this.setPage(1);
        break;

      case _types.Action.BACK:
        if (this.currentPage === 1) {
          if (this.loop) {
            this.setPage(this.pages.length);
          }

          return;
        }

        this.setPage(this.currentPage - 1);
        break;

      case _types.Action.STOP:
        this.stopListen();
        break;

      case _types.Action.NEXT:
        if (this.currentPage === this.pages.length) {
          if (this.loop) {
            this.setPage(1);
          }

          return;
        }

        this.setPage(this.currentPage + 1);
        break;

      case _types.Action.LAST:
        if (this.currentPage === this.pages.length) {
          if (this.loop) {
            this.setPage(1);
          }

          return;
        }

        this.setPage(this.pages.length);
        break;
    }
  }

}

exports.PagesBuilder = PagesBuilder;